// CREDIT: patriciogv
// ISADORA_PLUGIN_DESC("Generates a basic brick pattern")
// ISADORA_FLOAT_PARAM(brickSize, /+2U, 0, 1, 0.2, "No help available.")
// ISADORA_FLOAT_PARAM(fillSize, f/yG, 0, 1, 0.9, "No help available.")
// ISADORA_FLOAT_PARAM(brickOffset_x, TM8O, 0, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(brickOffset_y, 0-/z, 0, 1, 0, "No help available.")
// ISADORA_VEC4_COLOR_PARAM(fillColor, d+IY, 0x00000000, "No help available.")
// ISADORA_VEC4_COLOR_PARAM(brickColor, 1S-I, 0xFFFFFFFF, "No help available.")

#define brickOffset vec2(brickOffset_x,brickOffset_y)

uniform float brickSize;
uniform float fillSize;
uniform float brickOffset_x;
uniform float brickOffset_y;
uniform vec4 fillColor;
uniform vec4 brickColor;
uniform vec2 resolution;

// Based on Brick example from https://thebookofshaders.com/09/
// Author @patriciogv ( patriciogonzalezvivo.com ) - 2015



vec2 brickTile(vec2 _st, float _zoom){
    _st *= _zoom;

    // Here is where the offset is happening
    _st.x += step(1., mod(_st.y,2.0)) * 0.5;

    return fract(_st);
}

float box(vec2 _st, vec2 _size){
    _size = vec2(0.5)-_size*0.5;
    vec2 uv = smoothstep(_size,_size+vec2(1e-4),_st);
    uv *= smoothstep(_size,_size+vec2(1e-4),vec2(1.0)-_st);
    return uv.x*uv.y;
}

void main(void){
vec2 isf_FragNormCoord = gl_FragCoord.xy / resolution.xy;

    vec2 st = isf_FragNormCoord;
    vec4 color = fillColor;
    float brickCount = (brickSize == 0.0) ? max(resolution.x,resolution.y) : 1.0 / brickSize;
    //	Apply the offset
    st += brickOffset;
    // Apply the brick tiling
    st = brickTile(st,brickCount);

	float val = box(st,vec2(fillSize));
    color = mix(fillColor,brickColor,val);

    // Uncomment to see the space coordinates
    //color.rgb = vec3(st,0.0);

    gl_FragColor = color;
}

