// CREDIT: by carter rosenberg
// ISADORA_PLUGIN_DESC("Bends and distorts the image")
// ISADORA_FLOAT_PARAM(level, M*EO, -1, 1, 0.5, "No help available.")
// ISADORA_FLOAT_PARAM(radius, oHO5, 0, 1, 0.25, "No help available.")
// ISADORA_FLOAT_PARAM(center_x, tvT}, 0, 1, 0.5, "No help available.")
// ISADORA_FLOAT_PARAM(center_y, }%&g, 0, 1, 0.5, "No help available.")

#define center vec2(center_x,center_y)

uniform sampler2D tex0;
uniform float level;
uniform float radius;
uniform float center_x;
uniform float center_y;
uniform vec2 resolution;

const float pi = 3.14159265359;

#ifndef GL_ES
float distance (vec2 inCenter, vec2 pt)
{
	float tmp = pow(inCenter.x-pt.x,2.0)+pow(inCenter.y-pt.y,2.0);
	return pow(tmp,0.5);
}
#endif

void main() {
	vec2		uv = gl_TexCoord[0].xy;
	vec2		texSize = resolution.xy;
	vec2		tc = uv * texSize;
	vec2		modifiedCenter = center * texSize;
	float		r = distance(modifiedCenter, tc);
	float		a = atan ((tc.y-modifiedCenter.y),(tc.x-modifiedCenter.x));
	float		radius_sized = radius * length(resolution);
	
	tc -= modifiedCenter;

	if (r < radius_sized) 	{
		float percent = 1.0-(radius_sized - r) / radius_sized;
		if (level>=0.0)	{
			percent = percent * percent;
			tc.x = r*pow(percent,level) * cos(a);
			tc.y = r*pow(percent,level) * sin(a);
		}
		else	{
			float adjustedLevel = level/2.0;
			tc.x = r*pow(percent,adjustedLevel) * cos(a);
			tc.y = r*pow(percent,adjustedLevel) * sin(a);		
		}
	}
	tc += modifiedCenter;
	vec2 loc = tc / texSize;

	if ((loc.x < 0.0)||(loc.y < 0.0)||(loc.x > 1.0)||(loc.y > 1.0))	{
		gl_FragColor = vec4(0.0);
	}
	else	{
		gl_FragColor = texture2D(tex0, loc);
	}
}
