// CREDIT: VIDVOX
// ISADORA_PLUGIN_DESC("Wraps the video into a circular shape")
// ISADORA_FLOAT_PARAM(inputAngle, 9sLm, 0, 1, 0.5, "No help available.")
// ISADORA_FLOAT_PARAM(inputRadius, btXi, 0, 2, 1, "No help available.")
// ISADORA_FLOAT_PARAM(inputCenter_x, O&^4, 0, 1, 0.5, "No help available.")
// ISADORA_FLOAT_PARAM(inputCenter_y, P}18, 0, 1, 0.5, "No help available.")
// ISADORA_INT_PARAM_ONOFF(mirror, 1ssb, 0, "No help available.")
// ISADORA_INT_PARAM_ONOFF(correctAspect, $#4v, 0, "No help available.")

#define inputCenter vec2(inputCenter_x,inputCenter_y)

uniform sampler2D tex0;
uniform float inputAngle;
uniform float inputRadius;
uniform float inputCenter_x;
uniform float inputCenter_y;
uniform bool mirror;
uniform bool correctAspect;
uniform vec2 resolution;


const float tau = 6.28318530718;
const float pi = 3.14159265359;
const float halfpi = 1.57079632679;


void main()	{
	vec4		inputPixelColor;
	if (inputRadius > 0.0)	{
	
		vec2		loc = gl_TexCoord[0].xy;
		vec2		center = inputCenter;
	
		//	account for aspect ratio so we stay circle
		if (correctAspect)	{
			loc.x = loc.x * resolution.x/resolution.y;
			center.x = center.x * resolution.x/resolution.y;
		}
		//	translate from polar coords back to cart for this point
		//		the effect translates (x,y) to (r,theta)
		float		r = distance(loc,center);
		float		theta = (inputAngle * tau + pi + atan(loc.x-center.x,loc.y-center.y))/tau;
		
		theta = mod(theta, 1.0);
	
		//loc = vec2((r * 2.0)/inputRadius, theta / pi);
		loc = vec2(theta, (r * 2.0)/inputRadius);
		
		if ((loc.x < 0.0)||(loc.x > 1.0)||(loc.y < 0.0)||(loc.y > 1.0))
			inputPixelColor = vec4(0.0);
		else	{
			loc.y = 1.0 - loc.y;
			if (mirror)	{
				loc.x = (loc.x < 0.5) ? loc.x * 2.0 : 2.0 - loc.x * 2.0;
			}
			inputPixelColor = texture2D(tex0, loc);
		}
	}
	gl_FragColor = inputPixelColor;
}
