// CREDIT: by VIDVOX
// ISADORA_FLOAT_PARAM(width, ^c-[, 0, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(angle, +nv%, -0.5, 0.5, 0.125, "No help available.")


uniform sampler2D tex0;
uniform float width;
uniform float angle;
uniform vec2 resolution;


const float pi = 3.14159265359;


void main() {
	vec2 loc = gl_TexCoord[0].xy * resolution;

	//	for starters let's just do y = x * tan(angle) + b
	
	float b = 0.0;
	float tanVal = 0.0;
	vec2 p1 = vec2(0.0);
	vec2 p2 = vec2(1.0);
	
	if (abs(angle) != 0.5)	{
		tanVal = tan(pi * angle);
		b = (loc.y - loc.x * tanVal)/resolution.y;
		
		if (width > 0.0)	{
			float w = width * (1.0+abs(tanVal));
			b = w * floor(b/w);
		}
		b = b * resolution.y;
		
		//	if p1 is offscreen, adjust to an onscreen point
		p1 = vec2(0.0, b);
		
		//	in this case instead of using where it hits the left edge, use where it hits the bottom, solving x for y = 0.0 (x = (y - b) / tanVal)
		if (p1.y < 0.0)	{
			p1 = vec2(0.0-b / tanVal, 0.0);	
		}
		//	in this case instead of using where it hits the left edge, use where it hits the top, solving x for y = 1.0 (x = (y - b) / tanVal)
		else if (p1.y > resolution.y)	{
			p1 = vec2((resolution.x - b) / tanVal, resolution.y);
		}
	
		//	get the right side edge
		p2 = vec2(resolution.x, resolution.x * tanVal + b);
		
		//	if p2 is offscreen, adjust to an onscreen point
		//	in this case instead of using where it hits the right edge, use where it hits the bottom, solving x for y = 0.0 (x = (y - b) / tanVal)
		if (p2.y < 0.0)	{
			p2 = vec2(- b / tanVal, 0.0);
		}
		//	in this case instead of using where it hits the right edge, use where it hits the top, solving x for y = 1.0 (x = (y - b) / tanVal)
		else if (p2.y > resolution.y)	{
			p2 = vec2((resolution.y - b) / tanVal, resolution.y);
		}
		
	}
	//	vertical lines! set p1 & p2 to fixed x with y = 0.0 and 1.0
	else	{
		if (angle > 0.0)	{
			p1 = vec2(loc.x, 0.0);
			p2 = vec2(loc.x, resolution.y);
		}
		else	{
			p2 = vec2(loc.x, 0.0);
			p1 = vec2(loc.x, resolution.y);	
		}
		if (width > 0.0)	{
			p1.x = resolution.x * width * floor((loc.x/resolution.x) / width);
			p2.x = p1.x;
		}
	}
	
	//	now average 5 points on the line, p1, p2, their midpoint, and the mid points of those
	//	midpoint of the line
	vec2 mid = (p1 + p2) / 2.0;
	
	vec4 returnMe = (texture2D(tex0, (p1)/resolution) + texture2D(tex0, ((p1 + mid) / 2.0)/resolution) + texture2D(tex0, (mid)/resolution) + texture2D(tex0, ((p2 + mid) / 2.0)/resolution) + texture2D(tex0, (p2)/resolution)) / 5.0;
	//vec4 returnMe = texture2D(tex0, (p2)/resolution);

	gl_FragColor = returnMe;
}



