// CREDIT: VIDVOX
// ISADORA_PLUGIN_DESC("Stretches the edges out a region of the video")
// ISADORA_FLOAT_PARAM(leftEdge, V>J~, 0, 1, 0.25, "No help available.")
// ISADORA_FLOAT_PARAM(rightEdge, 6KvA, 0, 1, 0.75, "No help available.")
// ISADORA_FLOAT_PARAM(bottomEdge, ki02, 0, 1, 0.25, "No help available.")
// ISADORA_FLOAT_PARAM(topEdge, gN4], 0, 1, 0.75, "No help available.")
// ISADORA_INT_PARAM_ONOFF(doHorizontal, Q[&y, 0, "No help available.")
// ISADORA_INT_PARAM_ONOFF(doVertical, V2VR, 0, "No help available.")
// ISADORA_INT_PARAM_ONOFF(insideBleed, hiPe, 0, "No help available.")
// ISADORA_INT_PARAM_ONOFF(outsideBleed, c?Z~, 0, "No help available.")


uniform sampler2D tex0;
uniform float leftEdge;
uniform float rightEdge;
uniform float bottomEdge;
uniform float topEdge;
uniform bool doHorizontal;
uniform bool doVertical;
uniform bool insideBleed;
uniform bool outsideBleed;

void main()	{
	vec4		outputPixelColor;
	float		realLeftEdge = (leftEdge < rightEdge) ? leftEdge : rightEdge;
	float		realRightEdge = (leftEdge > rightEdge) ? leftEdge : rightEdge;
	
	float		realBottomEdge = (bottomEdge < topEdge) ? bottomEdge : topEdge;
	float		realTopEdge = (bottomEdge > topEdge) ? bottomEdge : topEdge;
	
	vec2		sampleCoord = gl_TexCoord[0].xy;

	bool		insideBox = false;
	
	vec2		region = vec2(0.0,0.0);
	
	if (sampleCoord.x < realLeftEdge)	{
		region.x = 0.0;
	}
	else if (sampleCoord.x > realRightEdge)	{
		region.x = 2.0;
	}
	else {
		region.x = 1.0;
	}
	
	if (sampleCoord.y < realBottomEdge)	{
		region.y = 0.0;
	}
	else if (sampleCoord.y > realTopEdge)	{
		region.y = 2.0;
	}
	else {
		region.y = 1.0;
	}
	
	if ((region.x == 1.0) && (region.y == 1.0) && (insideBleed == true))	{
		insideBox = true;
	}
	else if (outsideBleed == true)	{
		//	if we are in the bottom left...
		if ((region.x == 0.0) && (region.y == 0.0))	{
			if ((doHorizontal) && (doVertical))	{
				insideBox = true;
				realRightEdge = realLeftEdge;
				realLeftEdge = 0.0;
				realTopEdge = realBottomEdge;
				realBottomEdge = 0.0;
			}
		}
		else if ((region.x == 1.0) && (region.y == 0.0))	{
			if ((doHorizontal) && (doVertical))	{
				insideBox = true;
				realTopEdge = realBottomEdge;
				realBottomEdge = 0.0;
			}
			else if (doVertical)	{
				insideBox = true;
				realTopEdge = realBottomEdge;
				realBottomEdge = 0.0;
			}
		}
		else if ((region.x == 2.0) && (region.y == 0.0))	{
			if ((doHorizontal) && (doVertical))	{
				insideBox = true;
				realLeftEdge = realRightEdge;
				realRightEdge = 1.0;
				realTopEdge = realBottomEdge;
				realBottomEdge = 0.0;
			}
		}
		else if ((region.x == 0.0) && (region.y == 1.0))	{
			if (doHorizontal)	{
				insideBox = true;
				realRightEdge = realLeftEdge;
				realLeftEdge = 0.0;
			}
		}
		else if ((region.x == 2.0) && (region.y == 1.0))	{
			if (doHorizontal)	{
				insideBox = true;
				realLeftEdge = realRightEdge;
				realRightEdge = 1.0;
			}
		}
		else if ((region.x == 0.0) && (region.y == 2.0))	{
			if ((doHorizontal) && (doVertical))	{
				insideBox = true;
				realRightEdge = realLeftEdge;
				realLeftEdge = 0.0;
				realBottomEdge = realTopEdge;
				realTopEdge = 1.0;
			}
		}
		else if ((region.x == 1.0) && (region.y == 2.0))	{
			if (doVertical)	{
				insideBox = true;
				realBottomEdge = realTopEdge;
				realTopEdge = 1.0;
			}
		}
		else if ((region.x == 2.0) && (region.y == 2.0))	{
			if ((doHorizontal) && (doVertical))	{
				insideBox = true;
				realLeftEdge = realRightEdge;
				realRightEdge = 1.0;
				realBottomEdge = realTopEdge;
				realTopEdge = 1.0;
			}
		}
	}
	
	//	if we're doing inside bleed
	if (insideBox)	{
		
		//	how close are we to each edge?			
		if ((doHorizontal) && (doVertical))	{
			float	leftDistance = sampleCoord.x - realLeftEdge;
			float	rightDistance = realRightEdge - sampleCoord.x;
			float	bottomDistance = sampleCoord.y - realBottomEdge;
			float	topDistance = realTopEdge - sampleCoord.y;
			float	totalDistance = (leftDistance + rightDistance + bottomDistance + topDistance);
			
			vec4	leftPixel = texture2D(tex0, vec2(realLeftEdge, sampleCoord.y));
			vec4	rightPixel = texture2D(tex0, vec2(realRightEdge, sampleCoord.y));
			vec4	bottomPixel = texture2D(tex0, vec2(sampleCoord.x, realBottomEdge));
			vec4	topPixel = texture2D(tex0, vec2(sampleCoord.x, realTopEdge));
				
			outputPixelColor = (rightDistance * leftPixel + leftDistance * rightPixel + topDistance * bottomPixel + bottomDistance * topPixel) / totalDistance;
		}
		else if (doHorizontal)	{
			float	leftDistance = sampleCoord.x - realLeftEdge;
			float	rightDistance = realRightEdge - sampleCoord.x;
			float	totalDistance = leftDistance + rightDistance;
			vec4	leftPixel = texture2D(tex0, vec2(realLeftEdge, sampleCoord.y));
			vec4	rightPixel = texture2D(tex0, vec2(realRightEdge, sampleCoord.y));
			outputPixelColor = (rightDistance * leftPixel + leftDistance * rightPixel) / totalDistance;
		}
		else if (doVertical)	{
			float	bottomDistance = sampleCoord.y - realBottomEdge;
			float	topDistance = realTopEdge - sampleCoord.y;
			float	totalDistance = bottomDistance + topDistance;
			vec4	bottomPixel = texture2D(tex0, vec2(sampleCoord.x, realBottomEdge));
			vec4	topPixel = texture2D(tex0, vec2(sampleCoord.x, realTopEdge));
			outputPixelColor = (topDistance * bottomPixel + bottomDistance * topPixel) / totalDistance;	
		}
		else	{
			outputPixelColor = texture2D(tex0, sampleCoord);	
		}
	}
	else	{
		outputPixelColor = texture2D(tex0, sampleCoord);
	}
	
	gl_FragColor = outputPixelColor;
}
