// CREDIT: Automatically converted from https://www.github.com/gl-transitions/gl-transitions/tree/master/GridFlip.glsl
// ISADORA_FLOAT_PARAM(progress, zQT\, 0, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(pause, X^Gh, 0, 1, 0.1, "No help available.")
// ISADORA_FLOAT_PARAM(dividerWidth, q&>o, 0, 1, 0.05, "No help available.")
// ISADORA_FLOAT_PARAM(randomness, PH0m, 0, 1, 0.1, "No help available.")
// ISADORA_VEC4_COLOR_PARAM(bgcolor, N9Hy, 0xFF000000, "No help available.")
// ISADORA_FLOAT_PARAM(size_x, Lb?H, 0, 10, 4, "No help available.")
// ISADORA_FLOAT_PARAM(size_y, aWfI, 0, 10, 4, "No help available.")

#define size vec2(size_x,size_y)

uniform sampler2D tex0;
uniform sampler2D tex1;
uniform float progress;
uniform float pause;
uniform float dividerWidth;
uniform float randomness;
uniform vec4 bgcolor;
uniform float size_x;
uniform float size_y;



vec4 getFromColor(vec2 inUV)	{
	return texture2D(tex0, inUV);
}
vec4 getToColor(vec2 inUV)	{
	return texture2D(tex1, inUV);
}



// License: MIT
// Author: TimDonselaar
// ported by gre from https://gist.github.com/TimDonselaar/9bcd1c4b5934ba60087bdb55c2ea92e5

 
float rand (vec2 co) {
  return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

float getDelta(vec2 p) {
  vec2 rectanglePos = floor(vec2(size) * p);
  vec2 rectangleSize = vec2(1.0 / vec2(size).x, 1.0 / vec2(size).y);
  float top = rectangleSize.y * (rectanglePos.y + 1.0);
  float bottom = rectangleSize.y * rectanglePos.y;
  float left = rectangleSize.x * rectanglePos.x;
  float right = rectangleSize.x * (rectanglePos.x + 1.0);
  float minX = min(abs(p.x - left), abs(p.x - right));
  float minY = min(abs(p.y - top), abs(p.y - bottom));
  return min(minX, minY);
}

float getDividerSize() {
  vec2 rectangleSize = vec2(1.0 / vec2(size).x, 1.0 / vec2(size).y);
  return min(rectangleSize.x, rectangleSize.y) * dividerWidth;
}

vec4 transition(vec2 p) {
  if(progress < pause) {
    float currentProg = progress / pause;
    float a = 1.0;
    if(getDelta(p) < getDividerSize()) {
      a = 1.0 - currentProg;
    }
    return mix(bgcolor, getFromColor(p), a);
  }
  else if(progress < 1.0 - pause){
    if(getDelta(p) < getDividerSize()) {
      return bgcolor;
    } else {
      float currentProg = (progress - pause) / (1.0 - pause * 2.0);
      vec2 q = p;
      vec2 rectanglePos = floor(vec2(size) * q);
      
      float r = rand(rectanglePos) - randomness;
      float cp = smoothstep(0.0, 1.0 - r, currentProg);
    
      float rectangleSize = 1.0 / vec2(size).x;
      float delta = rectanglePos.x * rectangleSize;
      float offset = rectangleSize / 2.0 + delta;
      
      p.x = (p.x - offset)/abs(cp - 0.5)*0.5 + offset;
      vec4 a = getFromColor(p);
      vec4 b = getToColor(p);
      
      float s = step(abs(vec2(size).x * (q.x - delta) - 0.5), abs(cp - 0.5));
      return mix(bgcolor, mix(b, a, step(cp, 0.5)), s);
    }
  }
  else {
    float currentProg = (progress - 1.0 + pause) / pause;
    float a = 1.0;
    if(getDelta(p) < getDividerSize()) {
      a = currentProg;
    }
    return mix(bgcolor, getToColor(p), a);
  }
}



void main()	{
	gl_FragColor = transition(gl_TexCoord[0].xy);
}
