// CREDIT: by Carter Rosenberg
// ISADORA_FLOAT_PARAM(frequency1, pA9=, 0.5, 16, 1, "No help available.")
// ISADORA_FLOAT_PARAM(phase1, {~Z%, 0, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(amplitude1, Fnq6, -2, 2, 1, "No help available.")
// ISADORA_FLOAT_PARAM(offset1, >7Q^, -1, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(angle1, _3Cj, 0, 1, 0, "No help available.")
// ISADORA_INT_PARAM(curve1, ux/s, 0, 4, 0, "No help available.")
// ISADORA_FLOAT_PARAM(mixLevel1, QP2T, -1, 1, 1, "No help available.")
// ISADORA_VEC4_COLOR_PARAM(startColor1, ga8~, 0x00000000, "No help available.")
// ISADORA_VEC4_COLOR_PARAM(endColor1, ]j6l, 0xFFFF0000, "No help available.")
// ISADORA_FLOAT_PARAM(frequency2, lsBh, 0.5, 16, 1, "No help available.")
// ISADORA_FLOAT_PARAM(phase2, Io=H, 0, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(amplitude2, c5Vm, -2, 2, 1, "No help available.")
// ISADORA_FLOAT_PARAM(offset2, zoMW, -1, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(angle2, 8Qli, 0, 1, 0.75, "No help available.")
// ISADORA_INT_PARAM(curve2, &HYi, 0, 4, 0, "No help available.")
// ISADORA_FLOAT_PARAM(mixLevel2, <yT\, -1, 1, 1, "No help available.")
// ISADORA_VEC4_COLOR_PARAM(startColor2, C/mk, 0x00000000, "No help available.")
// ISADORA_VEC4_COLOR_PARAM(endColor2, nBTi, 0xFF00FF00, "No help available.")
// ISADORA_FLOAT_PARAM(frequency3, O9rN, 0.5, 16, 2, "No help available.")
// ISADORA_FLOAT_PARAM(phase3, $ry&, 0, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(amplitude3, &QDH, -2, 2, 1, "No help available.")
// ISADORA_FLOAT_PARAM(offset3, %Gy/, -1, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(angle3, -Gbv, 0, 1, 0, "No help available.")
// ISADORA_INT_PARAM(curve3, Y9^5, 0, 4, 0, "No help available.")
// ISADORA_FLOAT_PARAM(mixLevel3, GnEy, -1, 1, 1, "No help available.")
// ISADORA_VEC4_COLOR_PARAM(startColor3, /xWo, 0x00000000, "No help available.")
// ISADORA_VEC4_COLOR_PARAM(endColor3, 760z, 0xFF0000FF, "No help available.")


uniform sampler2D tex0;
uniform float frequency1;
uniform float phase1;
uniform float amplitude1;
uniform float offset1;
uniform float angle1;
uniform int curve1;
uniform float mixLevel1;
uniform vec4 startColor1;
uniform vec4 endColor1;
uniform float frequency2;
uniform float phase2;
uniform float amplitude2;
uniform float offset2;
uniform float angle2;
uniform int curve2;
uniform float mixLevel2;
uniform vec4 startColor2;
uniform vec4 endColor2;
uniform float frequency3;
uniform float phase3;
uniform float amplitude3;
uniform float offset3;
uniform float angle3;
uniform int curve3;
uniform float mixLevel3;
uniform vec4 startColor3;
uniform vec4 endColor3;
uniform vec2 resolution;



const float pi = 3.14159265359;
const float e = 2.71828182846;


float doMath(int curve, float freq, float phase, float val)	{
	float	returnMe = phase + freq * val;

	if (curve == 0)	{
		returnMe = mod(returnMe,1.0);
	}
	else if (curve == 1)	{
		returnMe = mod(2.0 * returnMe,2.0);
		returnMe = (returnMe < 1.0) ? returnMe : 1.0 - (returnMe - floor(returnMe));
	}
	else if (curve == 2)	{
		returnMe = sin(returnMe * pi * 2.0 - pi / 2.0) * 0.5 + 0.5;
	}
	else if (curve == 3)	{
		returnMe = mod(2.0 * returnMe, 2.0);
		returnMe = (returnMe < 1.0) ? returnMe : 1.0 - (returnMe - floor(returnMe));
		returnMe = pow(returnMe, 2.0);
	}
	else if (curve == 4)	{
		vec2	loc = mod(returnMe+gl_TexCoord[0].xy,1.0);
		vec4	tmp = texture2D(tex0, loc);
		returnMe = (tmp.r+tmp.g+tmp.b)*tmp.a/3.0;
	}
	return returnMe;	
}

//	note that this works on normalized points, but respects aspect ratio
vec2 rotatePoint(vec2 pt, float angle)	{
	vec2 returnMe = pt * resolution;;

	float r = distance(resolution/2.0, returnMe);
	float a = atan ((returnMe.y-resolution.y/2.0),(returnMe.x-resolution.x/2.0));

	returnMe.x = r * cos(a + 2.0 * pi * angle - pi) + 0.5;
	returnMe.y = r * sin(a + 2.0 * pi * angle - pi) + 0.5;
	
	returnMe = returnMe / resolution + vec2(0.5);
	
	return returnMe;
}


void main() {
	vec4 returnMe = vec4(0.0);
	vec4 blendColor = vec4(0.0);
	float mixAmount = 0.0;
	vec2 loc = gl_TexCoord[0].xy;

	loc = rotatePoint(gl_TexCoord[0].xy,angle1);
	mixAmount = doMath(curve1,frequency1,phase1,1.0-loc.x);
	mixAmount = (amplitude1 >= 0.0) ? mixAmount * amplitude1 : (1.0 - mixAmount) * abs(amplitude1);
	mixAmount += offset1;
	blendColor = mix(startColor1,endColor1,mixAmount);
	returnMe.rgb += blendColor.rgb * mixLevel1;
	returnMe.a += abs(blendColor.a);
	
	loc = rotatePoint(gl_TexCoord[0].xy,angle2);
	mixAmount = doMath(curve2,frequency2,phase2,1.0-loc.x);
	mixAmount = (amplitude2 >= 0.0) ? mixAmount * amplitude2 : (1.0 - mixAmount) * abs(amplitude2);
	mixAmount += offset2;
	blendColor = mix(startColor2,endColor2,mixAmount);
	returnMe.rgb += blendColor.rgb  * mixLevel2;
	returnMe.a += abs(blendColor.a);
	
	loc = rotatePoint(gl_TexCoord[0].xy,angle3);
	mixAmount = doMath(curve3,frequency3,phase3,loc.x);
	mixAmount = (amplitude3 >= 0.0) ? mixAmount * amplitude3 : (1.0 - mixAmount) * abs(amplitude3);
	mixAmount += offset3;
	blendColor = mix(startColor3,endColor3,mixAmount);
	returnMe.rgb += blendColor.rgb  * mixLevel3;
	returnMe.a += abs(blendColor.a);
	
	gl_FragColor = returnMe;
}
