// CREDIT: VIDVOX
// ISADORA_PLUGIN_DESC("Creates a stripe pattern with randomized colors")
// ISADORA_FLOAT_PARAM(width, ^c-[, 0, 1, 0.25, "No help available.")
// ISADORA_FLOAT_PARAM(offset, 7>D/, 0, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(hue, V%-n, 0, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(saturation, qh\y, 0, 1, 0.5, "No help available.")
// ISADORA_FLOAT_PARAM(brightness, +rn?, 0, 1, 0.95, "No help available.")
// ISADORA_INT_PARAM_ONOFF(vertical, |1G=, 0, "No help available.")
// ISADORA_INT_PARAM_ONOFF(randHue, $A%x, 1, "No help available.")
// ISADORA_INT_PARAM_ONOFF(randSaturation, uE{T, 0, "No help available.")
// ISADORA_INT_PARAM_ONOFF(randBright, LZ^{, 0, "No help available.")
// ISADORA_INT_PARAM_ONOFF(randAlpha, JGm5, 0, "No help available.")
// ISADORA_FLOAT_PARAM(rSeed, &pl#, 0, 1, 0.71, "No help available.")


uniform float width;
uniform float offset;
uniform float hue;
uniform float saturation;
uniform float brightness;
uniform bool vertical;
uniform bool randHue;
uniform bool randSaturation;
uniform bool randBright;
uniform bool randAlpha;
uniform float rSeed;
uniform vec2 resolution;

float rand(vec2 co){
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

vec4 rand4(vec4 co)	{
	vec4	returnMe = vec4(0.0);
	returnMe.r = rand(co.rg);
	returnMe.g = rand(co.gb);
	returnMe.b = rand(co.ba);
	returnMe.a = rand(co.rb);
	return returnMe;
}

vec3 hsv2rgb(vec3 c)	{
	vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
	vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
	return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {
vec2 isf_FragNormCoord = gl_FragCoord.xy / resolution.xy;


	vec4 out_color = vec4(0.0);
	float coord = (vertical) ? isf_FragNormCoord.x : isf_FragNormCoord.y;
	vec4 hsv = vec4(hue, saturation, brightness, 1.0);
	float minW = (vertical) ? max(width,1.0/resolution.x) : max(width,1.0/resolution.y);
	float index = floor((coord+offset) / minW);
	
	hsv.r = (randHue) ? mod(hsv.r + rand(vec2(index, rSeed + 0.34219)),1.0) : hsv.r;
	hsv.g = (randSaturation) ? mod(hsv.g + rand(vec2(index, rSeed + 0.57731)),1.0) : hsv.g;
	hsv.b = (randBright) ? mod(hsv.b + rand(vec2(index, rSeed + 0.79436)),1.0) : hsv.b;
	hsv.a = (randAlpha) ? rand(vec2(hsv.a + index, rSeed + 1.37665)) : hsv.a;
	
	out_color.rgb = hsv2rgb(hsv.rgb);
	out_color.a = hsv.a;
	
	gl_FragColor = out_color;
}
