// ISADORA_FLOAT_PARAM(startSize, Yp#9, 0, 2, 0.5, "No help available.")
// ISADORA_FLOAT_PARAM(startOpacity, &pvP, 0, 1, 1, "No help available.")
// ISADORA_FLOAT_PARAM(startCenter_x, qrSc, -1000000, +1000000, 0, "No help available.")
// ISADORA_FLOAT_PARAM(startCenter_y, |[gU, -1000000, +1000000, 0, "No help available.")
// ISADORA_FLOAT_PARAM(startPadding, QHMU, 0, 1, 0.25, "No help available.")
// ISADORA_FLOAT_PARAM(endSize, [4rU, 0, 2, 0.25, "No help available.")
// ISADORA_FLOAT_PARAM(endOpacity, 7Xxu, 0, 1, 1, "No help available.")
// ISADORA_FLOAT_PARAM(endCenter_x, |mEJ, -1000000, +1000000, 0, "No help available.")
// ISADORA_FLOAT_PARAM(endCenter_y, 4LZ>, -1000000, +1000000, 0, "No help available.")
// ISADORA_FLOAT_PARAM(endPadding, o=-6, 0, 1, 0.1, "No help available.")
// ISADORA_FLOAT_PARAM(repetitions, ^6hT, 1, 15, 5, "No help available.")

#define startCenter vec2(startCenter_x,startCenter_y)
#define endCenter vec2(endCenter_x,endCenter_y)

uniform sampler2D tex0;
uniform float startSize;
uniform float startOpacity;
uniform float startCenter_x;
uniform float startCenter_y;
uniform float startPadding;
uniform float endSize;
uniform float endOpacity;
uniform float endCenter_x;
uniform float endCenter_y;
uniform float endPadding;
uniform float repetitions;
uniform vec2 resolution;


vec2 paddedZoomedPosition(vec2 loc, float zl, vec2 c, float p)	{
	vec2		returnMe = loc;
	float		zoomMult = (1.0/zl);
	vec2		modifiedCenter = 2.0*(1.0+p)*c/resolution-(1.0+p);
	float		modifiedPadding = p;
	
	returnMe.x = (returnMe.x)*zoomMult + p/2.0 - modifiedCenter.x;
	returnMe.y = (returnMe.y)*zoomMult + p/2.0 - modifiedCenter.y;
	returnMe.x = mod(returnMe.x,1.0+modifiedPadding) - p/2.0;
	returnMe.y = mod(returnMe.y,1.0+modifiedPadding) - p/2.0;
	
	return returnMe;
}


void main()	{
	vec4		inputPixelColor = vec4(0.0);

	int			depth = int(repetitions);
	vec2		loc = gl_TexCoord[0].xy;
	float		minZoomLevel = (1.0/resolution.x);
	float		startZoomLevel = (startSize < minZoomLevel) ?  minZoomLevel : startSize;
	float		endZoomLevel = (endSize < minZoomLevel) ?  minZoomLevel : endSize;
	float		zoomIncrement = (depth < 2) ? 0.0 : (endZoomLevel - startZoomLevel)/float(depth-1);
	vec2		centerIncrement = (depth < 2) ? vec2(0.0) : (endCenter - startCenter)/float(depth-1);
	float		paddingIncrement = (depth < 2) ? 0.0 : (endPadding - startPadding)/float(depth-1);
	float		opacityIncrement = (depth < 2) ? 0.0 : (endOpacity - startOpacity)/float(depth-1);
	
	for (int i = 0;i < 15;++i)	{
		if (i >= depth)
			break;
		float	modZoom = startZoomLevel + zoomIncrement * float(i);
		vec2	modCenter = startCenter + centerIncrement * float(i);
		float	modPad = startPadding + paddingIncrement * float(i);
		float	modOpacity = startOpacity + opacityIncrement * float(i);
		modOpacity = clamp(modOpacity,0.0,1.0);
		loc = paddedZoomedPosition(gl_TexCoord[0].xy,modZoom,modCenter,modPad);
		if ((loc.x < 0.0)||(loc.y < 0.0)||(loc.x > 1.0)||(loc.y > 1.0))	{
			//inputPixelColor = vec4(0.0);
		}
		else	{
			vec4	tmpColor = texture2D(tex0, loc);
			inputPixelColor.rgb = inputPixelColor.rgb + tmpColor.rgb * tmpColor.a * modOpacity;
			inputPixelColor.a += (tmpColor.a * modOpacity);
			if (inputPixelColor.a > 0.99)	{
				break;
			}
		}
	}
	
	gl_FragColor = inputPixelColor;
}
