// CREDIT: by VIDVOX
// ISADORA_PLUGIN_DESC("Pixels update only if within range of the specified lines to create a slit scan style")
// ISADORA_FLOAT_PARAM(spacing, o+HH, 0, 1.5, 1, "No help available.")
// ISADORA_FLOAT_PARAM(line_width, EV{u, 0, 1, 0.33, "No help available.")
// ISADORA_FLOAT_PARAM(angle, +nv%, -1, 1, 0.25, "No help available.")
// ISADORA_FLOAT_PARAM(shift, S#J0, 0, 1, 0.5, "No help available.")
// ISADORA_FLOAT_PARAM(edgeSharpness, 3&UP, 0, 1, 0, "No help available.")


uniform sampler2D tex0;
uniform float spacing;
uniform float line_width;
uniform float angle;
uniform float shift;
uniform float edgeSharpness;
uniform vec2 resolution;


const float pi = 3.14159265359;


float pattern() {
	float s = sin(angle * pi);
	float c = cos(angle * pi);
	vec2 tex = gl_TexCoord[0].xy * resolution;
	float spaced = length(resolution) * spacing;
	vec2 point = vec2( c * tex.x - s * tex.y, s * tex.x + c * tex.y ) * max(1.0/spaced,0.001);
	float d = point.y;
	float w = line_width;
	if (w > spacing)	{
		w = 0.99*spacing;	
	}
	return ( mod(d + shift*spacing + w * 0.5,spacing) );
}

void main()
{
	vec4	freshPixel = texture2D(tex0, (gl_FragCoord.xy)/resolution);
	//	If we're on the line, update, otherwise use the stale pixel
	//vec4 	result = IMG_PIXEL(bufferVariableNameA,gl_FragCoord.xy);
	vec4 result = vec4(0.0);
	float pat = pattern();
	float w = line_width;
	if (w > spacing)	{
		w = 0.99*spacing;	
	}

	if ((pat > 0.0)&&(pat <= w))	{
		float percent = (1.0-abs(w-2.0*pat)/w);
		percent = clamp(percent,0.0,1.0);
		percent = mix(percent, pow(percent,1.0/5.0), edgeSharpness);
		result = mix(result, freshPixel, percent);
		//result = vec4(percent,percent,percent,1.0);
		//result = clamp(result+edgeSharpness, 0.0, 1.0);
	}
	gl_FragColor = result;
}
