// CREDIT: Automatically converted from https://www.github.com/gl-transitions/gl-transitions/tree/master/swap.glsl
// ISADORA_FLOAT_PARAM(progress, zQT\, 0, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(perspective, {z0A, 0, 1, 0.2, "No help available.")
// ISADORA_FLOAT_PARAM(depth, Z0=n, 0, 10, 3, "No help available.")
// ISADORA_FLOAT_PARAM(reflection, SXNS, 0, 1, 0.4, "No help available.")


uniform sampler2D tex0;
uniform sampler2D tex1;
uniform float progress;
uniform float perspective;
uniform float depth;
uniform float reflection;



vec4 getFromColor(vec2 inUV)	{
	return texture2D(tex0, inUV);
}
vec4 getToColor(vec2 inUV)	{
	return texture2D(tex1, inUV);
}



// Author: gre
// License: MIT
// General parameters
 
const vec4 black = vec4(0.0, 0.0, 0.0, 1.0);
const vec2 boundMin = vec2(0.0, 0.0);
const vec2 boundMax = vec2(1.0, 1.0);
 
bool inBounds (vec2 p) {
  return all(lessThan(boundMin, p)) && all(lessThan(p, boundMax));
}
 
vec2 project (vec2 p) {
  return p * vec2(1.0, -1.2) + vec2(0.0, -0.02);
}
 
vec4 bgColor (vec2 p, vec2 pfr, vec2 pto) {
  vec4 c = black;
  pfr = project(pfr);
  if (inBounds(pfr)) {
    c += mix(black, getFromColor(pfr), reflection * mix(1.0, 0.0, pfr.y));
  }
  pto = project(pto);
  if (inBounds(pto)) {
    c += mix(black, getToColor(pto), reflection * mix(1.0, 0.0, pto.y));
  }
  return c;
}
 
vec4 transition(vec2 p) {
  vec2 pfr, pto = vec2(-1.);
 
  float size = mix(1.0, depth, progress);
  float persp = perspective * progress;
  pfr = (p + vec2(-0.0, -0.5)) * vec2(size/(1.0-perspective*progress), size/(1.0-size*persp*p.x)) + vec2(0.0, 0.5);
 
  size = mix(1.0, depth, 1.-progress);
  persp = perspective * (1.-progress);
  pto = (p + vec2(-1.0, -0.5)) * vec2(size/(1.0-perspective*(1.0-progress)), size/(1.0-size*persp*(0.5-p.x))) + vec2(1.0, 0.5);

  if (progress < 0.5) {
    if (inBounds(pfr)) {
      return getFromColor(pfr);
    }
    if (inBounds(pto)) {
      return getToColor(pto);
    }  
  }
  if (inBounds(pto)) {
    return getToColor(pto);
  }
  if (inBounds(pfr)) {
    return getFromColor(pfr);
  }
  return bgColor(p, pfr, pto);
}



void main()	{
	gl_FragColor = transition(gl_TexCoord[0].xy);
}
