// CREDIT: by VIDVOX
// ISADORA_FLOAT_PARAM(normalEdgeThreshold, twXf, 0, 0.1, 0.03, "No help available.")
// ISADORA_FLOAT_PARAM(qLevel, _~-n, 2, 64, 32, "No help available.")


uniform sampler2D tex0;
uniform float normalEdgeThreshold;
uniform float qLevel;
uniform vec2 resolution;

//	with help from https://github.com/neilmendoza/ofxPostProcessing/blob/master/src/ToonPass.cpp


vec3 getNormal(vec2 st){
	vec2 texcoord = clamp(st, 0.001, 0.999);
	return texture2D(tex0, texcoord).rgb;
}

void main(void){
	float dxtex = 1.0 / resolution.x;
	float dytex = 1.0 / resolution.y;

	vec2 st = vec2(gl_TexCoord[0].xy[0],gl_TexCoord[0].xy[1]);
	// access center pixel and 4 surrounded pixel
	vec3 center = getNormal(st).rgb;
	vec3 left = getNormal(st + vec2(dxtex, 0.0)).rgb;
	vec3 right = getNormal(st + vec2(-dxtex, 0.0)).rgb;
	vec3 up = getNormal(st + vec2(0.0, -dytex)).rgb;
	vec3 down = getNormal(st + vec2(0.0, dytex)).rgb;

	// discrete Laplace operator
	vec3 laplace = abs(-4.0*center + left + right + up + down);
	// if one rgb-component of convolution result is over threshold => edge
	vec4 line = texture2D(tex0, st);
	if(laplace.r > normalEdgeThreshold
	|| laplace.g > normalEdgeThreshold
	|| laplace.b > normalEdgeThreshold){
		line = vec4(0.0, 0.0, 0.0, 1.0); // => color the pixel green
	} else {
		line = vec4(1.0, 1.0, 1.0, 1.0); // black
	}

	//end Line;
	//gl_FragColor = line;
	
	vec4 color = texture2D(tex0, gl_TexCoord[0].xy);

	// store previous alpha value
	float alpha = color.a;
	// quantize process: multiply by factor, round and divde by factor
	color = floor((qLevel * color)) / qLevel;
	// set fragment/pixel color
	color.a = alpha;

	gl_FragColor = color * line;
	
}

