// CREDIT: by VIDVOX
// ISADORA_PLUGIN_DESC("Performs three different rotations")
// ISADORA_FLOAT_PARAM(angle1, _3Cj, 0, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(angle2, 8Qli, 0, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(angle3, -Gbv, 0, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(angle4, yG>v, 0, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(radius1, bP5y, 0, 1, 0.15, "No help available.")
// ISADORA_FLOAT_PARAM(radius2, <[YK, 0, 1, 0.15, "No help available.")
// ISADORA_FLOAT_PARAM(radius3, ^MPg, 0, 1, 0.15, "No help available.")


uniform sampler2D tex0;
uniform float angle1;
uniform float angle2;
uniform float angle3;
uniform float angle4;
uniform float radius1;
uniform float radius2;
uniform float radius3;
uniform vec2 resolution;

const float pi = 3.14159265359;

void main()	{
	//	'loc' is the location in pixels of this vertex.  we're going to convert this to polar coordinates (radius/angle)
	vec2		loc = resolution * vec2(gl_TexCoord[0].xy[0],gl_TexCoord[0].xy[1]);
	//	'r' is the radius- the distance in pixels from 'loc' to the center of the rendering space
	float		r = distance(resolution/2.0, loc);
	//	'a' is the angle of the line segment from the center to loc is rotated
	float		a = atan ((loc.y-resolution.y/2.0),(loc.x-resolution.x/2.0));
	
	//	now modify 'a', and convert the modified polar coords (radius/angle) back to cartesian coords (x/y pixels)
	float	angle = angle1;
	float	minSide = min(resolution.x,resolution.y);
	if (r > (radius1 + radius2 + radius3)*minSide)
		angle = angle4;
	else if (r > (radius1 + radius2)*minSide)
		angle = angle3;
	else if (r > radius1 * minSide)
		angle = angle2;
	loc.x = r * cos(a + 2.0 * pi * angle);
	loc.y = r * sin(a + 2.0 * pi * angle);
	
	loc = loc / resolution + vec2(0.5);
	
	if ((loc.x < 0.0)||(loc.y < 0.0)||(loc.x > 1.0)||(loc.y > 1.0))	{
		gl_FragColor = vec4(0.0);
	}
	else	{
		gl_FragColor = texture2D(tex0, loc);
	}
}
