// CREDIT: David Lublin, original by Staffan Widegarn Åhlvik
// ISADORA_PLUGIN_DESC("VHS Glitch Style")
// ISADORA_INT_PARAM_ONOFF(autoScan, W9Pf, 1, "No help available.")
// ISADORA_FLOAT_PARAM(xScanline, a/lk, 0, 1, 0.5, "No help available.")
// ISADORA_FLOAT_PARAM(xScanline2, 3sQ9, 0, 1, 0.5, "No help available.")
// ISADORA_FLOAT_PARAM(yScanline, LEYP, 0, 1, 0, "No help available.")
// ISADORA_FLOAT_PARAM(xScanlineSize, zef>, 0, 1, 0.5, "No help available.")
// ISADORA_FLOAT_PARAM(xScanlineSize2, 9A-e, 0, 1, 0.25, "No help available.")
// ISADORA_FLOAT_PARAM(yScanlineAmount, 2|Tn, -1, 1, 0.25, "No help available.")
// ISADORA_FLOAT_PARAM(grainLevel, 1Am<, 0, 3, 0, "No help available.")
// ISADORA_INT_PARAM_ONOFF(scanFollow, crt5, 1, "No help available.")
// ISADORA_FLOAT_PARAM(analogDistort, M^ur, 0, 10, 1, "No help available.")
// ISADORA_FLOAT_PARAM(bleedAmount, PT<A, 0, 3, 1, "No help available.")
// ISADORA_FLOAT_PARAM(bleedDistort, #|qo, 0, 1, 0.5, "No help available.")
// ISADORA_FLOAT_PARAM(bleedRange, amZS, 0, 2, 1, "No help available.")
// ISADORA_VEC4_COLOR_PARAM(colorBleedL, CO8|, 0xFFCC0066, "No help available.")
// ISADORA_VEC4_COLOR_PARAM(colorBleedC, pcIe, 0xFF007FBF, "No help available.")
// ISADORA_VEC4_COLOR_PARAM(colorBleedR, c|os, 0xFFCC0066, "No help available.")


uniform sampler2D tex0;
uniform bool autoScan;
uniform float xScanline;
uniform float xScanline2;
uniform float yScanline;
uniform float xScanlineSize;
uniform float xScanlineSize2;
uniform float yScanlineAmount;
uniform float grainLevel;
uniform bool scanFollow;
uniform float analogDistort;
uniform float bleedAmount;
uniform float bleedDistort;
uniform float bleedRange;
uniform vec4 colorBleedL;
uniform vec4 colorBleedC;
uniform vec4 colorBleedR;
uniform vec2 resolution;
uniform float time;




//	Based on https://github.com/staffantan/unity-vhsglitch
//	Converted by David Lublin / VIDVOX


const float tau = 6.28318530718;


float rand(vec3 co){
	return abs(mod(sin( dot(co.xyz ,vec3(12.9898,78.233,45.5432) )) * 43758.5453, 1.0));
}

void main()	{
	float	actualXLine = (!autoScan) ? xScanline : mod(xScanline + ((1.0+sin(0.34*time))/2.0 + (1.0+sin(time))/3.0 + (1.0+cos(2.1*time))/3.0 + (1.0+cos(0.027*time))/2.0)/3.5,1.0);
	float	actualXLineWidth = (!autoScan) ? xScanlineSize : xScanlineSize + ((1.0+sin(1.2*time))/2.0 + (1.0+cos(3.91*time))/3.0 + (1.0+cos(0.014*time))/2.0)/3.5;
	vec2	loc = gl_TexCoord[0].xy;
	vec4	vhs = texture2D(tex0, loc);
	float	dx = 1.0+actualXLineWidth/25.0-abs(distance(loc.y, actualXLine));
	float	dx2 = 1.0+xScanlineSize2/10.0-abs(distance(loc.y, xScanline2));
	float	dy = (1.0-abs(distance(loc.y, yScanline)));
	if (autoScan)
		dy = (1.0-abs(distance(loc.y, mod(yScanline+time,1.0))));
	
	dy = (dy > 0.5) ? 2.0 * dy : 2.0 * (1.0 - dy);
	
	float	rX = (scanFollow) ? rand(vec3(dy,actualXLine,analogDistort)) : rand(vec3(dy,bleedAmount,analogDistort));
	float	xTime = (actualXLine > 0.5) ? 2.0 * actualXLine : 2.0 * (1.0 - actualXLine);
	
	loc.x += yScanlineAmount * dy * 0.025 + analogDistort * rX/(resolution.x/2.0);
	
	if(dx2 > 1.0 - xScanlineSize2 / 10.0)	{
		float	rX2 = (dy * rand(vec3(dy,dx2,dx+time)) + dx2) / 4.0;
		float	distortAmount = analogDistort * (sin(rX * tau / dx2) + cos(rX * tau * 0.78 / dx2)) / 10.0;
		//loc.y = xScanline2;
		//loc.x += (1.0 + distortAmount * sin(tau * (loc.x) / rX2 ) - 1.0) / 15.0;
		loc.x += (1.0 + distortAmount * sin(tau * (loc.x) / rX2 ) - 1.0) / 15.0;
	}
	if(dx > 1.0 - actualXLineWidth / 25.0)
		loc.y = actualXLine;

	loc.x = mod(loc.x,1.0);
	loc.y = mod(loc.y,1.0);
	
	vec4	c = texture2D(tex0, loc);
	float	x = (loc.x*320.0)/320.0;
	float	y = (loc.y*240.0)/240.0;
	float	bleed = 0.0;
	
	if (scanFollow)
		c -= rand(vec3(x, y, xTime)) * xTime / (5.0-grainLevel);
	else
		c -= rand(vec3(x, y, bleedAmount)) * (bleedAmount/20.0) / (5.0-grainLevel);
	
	if (bleedAmount > 0.0)	{
		texture2D(tex0, loc + vec2(0.01, 0)).r;
		bleed += texture2D(tex0, loc + bleedRange * vec2(0.02, 0)).r;
		bleed += texture2D(tex0, loc + bleedRange * vec2(0.01, 0.01)).r;
		bleed += texture2D(tex0, loc + bleedRange * vec2(-0.02, 0.02)).r;
		bleed += texture2D(tex0, loc + bleedRange * vec2(0.0, -0.03)).r;
		bleed /= 6.0;
		bleed *= bleedAmount;
	}

	if (bleed > 0.1){
		float	bleedFreq = 1.0;
		float	bleedX = 0.0;
		if (autoScan)
			bleedX = x + bleedDistort * (yScanlineAmount + (1.5 + cos(time / 13.0 + tau*(bleedDistort+(1.0-loc.y))))/2.0) * sin((time / 9.0 + bleedDistort) * tau + loc.y * loc.y * tau * bleedFreq) / 8.0;
		else
			bleedX = x + (yScanlineAmount + (1.0 + sin(tau*(bleedDistort+loc.y)))/2.0) * sin(bleedDistort * tau + loc.y * loc.y * tau * bleedFreq) / 10.0;
		vec4	colorBleed = (bleedX < 0.5) ? mix(colorBleedL, colorBleedC, 2.0 * bleedX) : mix(colorBleedR, colorBleedC, 2.0 - 2.0 * bleedX);
		if (scanFollow)
			c += bleed * max(xScanlineSize,xTime) * colorBleed;
		else
			c += bleed * colorBleed;
	}
	gl_FragColor = c;
}
